/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a modal window with the list of all jobs and prompts the user to select a job.
 * Upon job selection, the selected JobDetail is passed to the success handler if defined.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SelectJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.Const',

    'QDesk.util.quartz.QuartzUtils',

    'QDesk.view.main.scheduler.quartz.SelectJobWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

//    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectJob.maskLoadMsg' ) );

    var window = Ext.widget( 'QuartzSelectJobWindow', {
      qdesk : {
        connectionId : me.connectionId
      }
    } );

    me._bindEventHandlers( window );

    window.show();
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );
    var selectJobGrid = formPanel.down( '#selectJobGrid' );
    var actionButton = formPanel.down( '#actionButton' );

    // the action button is disabled by default
    actionButton.disable();

    /*
     * Start of selectJobGrid toolbar event handlers.
     */

    // refresh button
    selectJobGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshSelectJobGrid( selectJobGrid );
        } );

    var filterByJobGroupCombo = selectJobGrid.down( '#filterByJobGroupCombo' );
    filterByJobGroupCombo.on( 'change', function ( combo, newValue )
    {
      selectJobGrid.getStore().clearFilter( true );

      if ( newValue != null )
      {
        selectJobGrid.getStore().addFilter( [
          Ext.create( 'Ext.util.Filter', {
            property : 'group.name',
            value : newValue
          } )
        ], false );
      }

      // apply, or clear the filter and load the first page
      selectJobGrid.getStore().loadPage( 1 );
    } );

    /*
     * End of selectJobGrid toolbar event handlers.
     */

    selectJobGrid.getStore().on( 'beforeload',
        function ( store, operation )
        {
          selectJobGrid.down( '#refreshButton' ).disable();
          filterByJobGroupCombo.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          selectJobGrid.getSelectionModel().deselectAll();
        } );

    selectJobGrid.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          selectJobGrid.down( '#refreshButton' ).enable();
          if ( successful )
          {
            filterByJobGroupCombo.enable();
          }
          else
          {
            store.removeAll();

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectJob.errorLoadingJobsMsg' ) );
          }
        } );

    // job selection
    selectJobGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          actionButton.setDisabled( selectedRecords.length == 0 );
        } );


    // Save/Update button
    actionButton.on( 'click',
        function ()
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            if ( selectJobGrid.getSelectionModel().hasSelection() )
            {
              var jobDetail = selectJobGrid.getSelectionModel().getSelection()[0];

              window.close();
              me.success.apply( me, [jobDetail] );
            }
          }
        } );

    window.on( 'afterrender',
        function ()
        {
          me._refreshSelectJobGrid( selectJobGrid );
        } );
  },


  _refreshSelectJobGrid : function ( selectJobGrid )
  {
    var me = this;

    var filterByJobGroupCombo = selectJobGrid.down( '#filterByJobGroupCombo' );

    // reloads and resets the job group filter combo and then reloads the jobs grid
    filterByJobGroupCombo.getStore().load( {
      scope : me,
      callback : function ( records, operation, success )
      {
        if ( success )
        {
          // reload the jobs grid
          if ( filterByJobGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            selectJobGrid.getStore().loadPage( 1 );
          }
          else
          {
            // triggers the change event which reloads the jobs grid
            filterByJobGroupCombo.reset( null );
          }
        }
        else
        {
          //
        }
      }
    } );
  }
} );
