/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.SelectJobGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.store.quartz.JobDetailStore',
    'QDesk.store.quartz.JobGroupStore',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzSelectJobGrid',

  cls : 'quartzSelectJobGrid',

  viewConfig : {
    stripeRows : true
  },

  autoScroll : true,

  //border : false,

  qdesk : {
    /*
     * Scheduler connection ID.
     */
    connectionId : undefined
  },


  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
            '<tpl if="!description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
            '</p>'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var jobDetailStore = Ext.create( 'QDesk.store.quartz.JobDetailStore' );

    jobDetailStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : jobDetailStore,

      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        }
      ],

      columns : [
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textGroupNameColumn' ),
          dataIndex : 'group.name',
          tpl : '{group.name}',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.SelectJobGrid.textJobClassColumn' ),
          dataIndex : 'jobClass',
          sortable : true,
          groupable : true,
          flex : 1
        }
      ]
    } );

    me.callParent( [config] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.quartz.JobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;
  }
} );
