/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.triggers.JobTriggersGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.TriggerType',

    'QDesk.store.quartz.JobTriggerStore',
    'QDesk.store.quartz.TriggerGroupStore',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzJobTriggersGrid',

  cls : 'quartzJobTriggersGrid',

  autoScroll : true,

  //border : false,


  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.xy );
    }
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobTriggerGroupStore = Ext.create( 'QDesk.store.quartz.JobTriggerGroupStore' );
    jobTriggerGroupStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;

    me.down( '#filterByTriggerGroupCombo' ).store = jobTriggerGroupStore;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var schedulerVersion = config.qdesk.schedulerInfo.version;
    var jvmAgentAvailableAndCompatible = config.qdesk.jvmAgentInfo.compatible;

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resumeTriggerButton',
          scale : 'small',
          iconCls : 'resumeTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipResumeTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerButton',
          scale : 'small',
          iconCls : 'pauseTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipPauseTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'button',
          itemId : 'addTriggerButton',
          scale : 'small',
          iconCls : 'addTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipAddTriggerButton' ),
          menu : {
            items : QDesk.util.quartz.QuartzTriggerUtils.createAddTriggerButtonMenuItems( schedulerVersion )
          },
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'deleteTriggerButton',
          scale : 'small',
          iconCls : 'deleteTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipDeleteTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'editTriggerButton',
          scale : 'small',
          iconCls : 'editTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipEditTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'cloneTriggerButton',
          scale : 'small',
          iconCls : 'cloneTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipCloneTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.filterByJobGroupComboLabel' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'resumeTriggerGroupButton',
          scale : 'small',
          iconCls : 'resumeTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipResumeTriggerGroupButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerGroupButton',
          scale : 'small',
          iconCls : 'pauseTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipPauseTriggerGroupButton' ),
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ]
    } );

    Ext.apply( config, {
      columns : [
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textGroupNameColumn' ),
          dataIndex : 'group.name',
          tpl : '{group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'typeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textTypeColumn' ),
          dataIndex : 'type',
          tpl : '{[Ext.isEmpty( values.type ) ? QDesk.Msg.get("QDesk.unknown") : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( values.type )]}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'stateColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textStateColumn' ),
          dataIndex : 'state',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipJobDataMapColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var triggerRecord = grid.getStore().getAt( rowIndex );

                var jobDataMap = triggerRecord.get( 'jobDataMap' );
                var effectiveJobDataMap = triggerRecord.get( 'effectiveJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap, effectiveJobDataMap );
              }
            }
          ]
        },
        {
          xtype : 'datecolumn',
          itemId : 'startTimeColumn',
          format : QDesk.util.constant.DateFormat.timestampFormatLong,
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textStartTimeColumn' ),
          dataIndex : 'startTime',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'endTimeColumn',
          format : QDesk.util.constant.DateFormat.timestampFormatLong,
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textEndTimeColumn' ),
          dataIndex : 'endTime',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'previousFireTimeColumn',
          format : QDesk.util.constant.DateFormat.timestampFormatLong,
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textPreviousFireTimeColumn' ),
          dataIndex : 'previousFireTime',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'nextFireTimeColumn',
          format : QDesk.util.constant.DateFormat.timestampFormatLong,
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textNextFireTimeColumn' ),
          dataIndex : 'nextFireTime',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'finalFireTimeColumn',
          format : QDesk.util.constant.DateFormat.timestampFormatLong,
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textFinalFireTimeColumn' ),
          dataIndex : 'finalFireTime',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'calendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textCalendarNameColumn' ),
          dataIndex : 'calendar',
          tpl : '{calendar.displayName}',
          sortable : false,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'numbercolumn',
          itemId : 'priorityColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textPriorityColumn' ),
          dataIndex : 'priority',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'numbercolumn',
          itemId : 'misfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.textMisfireInstructionColumn' ),
          dataIndex : 'misfireInstruction',
          sortable : false,
          groupable : false,
          flex : 1
        }
      ]
    } );

    Ext.apply( config, {
      plugins : [
        {
          ptype : 'rowexpander',
          rowBodyTpl : [
            '<p>' +
                '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
                '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
                '<tpl if="!description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
                '</p>' +

                '{[QDesk.util.quartz.QuartzTriggerUtils.createTriggerPropertiesTable( values )]}' +

                '<p style="white-space: nowrap">' +
                '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
                '{[this.getMonitoringLink( values.monitoringUrl )]}' +
                '</p>',
            {
              getMonitoringLink : function ( monitoringUrl )
              {
                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
                var tooltip;
                if ( licensedFeatures.TriggerMonitoring )
                {
                  tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipMonitoringUrl' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
                }
                else
                {
                  tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.tooltipMonitoringUrlNotLicensed' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                      tooltip + '">' + monitoringUrl + '</a>';
                }
              }
            }
          ]
        }
      ]
    } );

    var jobTriggerStore = Ext.create( 'QDesk.store.quartz.JobTriggerStore' );

    Ext.apply( config, {
      store : jobTriggerStore,

      qdesk : {
        gridId : 'quartz.jobTriggersGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [config] );
  },


  /**
   * Invoked to create a context menu for the specified trigger in the JobTriggersGrid row.
   * @param trigger a trigger.
   * @private
   */
  _createRowCtxMenu : function ( trigger )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // resume trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuResume' ),
          iconCls : 'resumeTrigger',
          disabled : trigger.get( 'state' ) != QDesk.enums.scheduler.quartz.TriggerState.PAUSED,
          handler : function ()
          {
            me.fireEvent( 'resumeTrigger', trigger );
          }
        },
        // pause trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuPause' ),
          iconCls : 'pauseTrigger',
          disabled : trigger.get( 'state' ) != QDesk.enums.scheduler.quartz.TriggerState.NORMAL,
          handler : function ()
          {
            me.fireEvent( 'pauseTrigger', trigger );
          }
        },
        '-',
        // delete trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuDelete' ),
          iconCls : 'deleteTrigger',
          handler : function ()
          {
            me.fireEvent( 'deleteTrigger', trigger );
          }
        },
        // edit trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuEdit' ),
          iconCls : 'editTrigger',
          disabled : !QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.qdesk.connectionId ),
          handler : function ()
          {
            me.fireEvent( 'editTrigger', trigger );
          }
        },
        // clone trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuClone' ),
          iconCls : 'cloneTrigger',
          disabled : !QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.qdesk.connectionId ),
          handler : function ()
          {
            me.fireEvent( 'cloneTrigger', trigger );
          }
        }
      ]
    } );
  }
} );
