/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart', {
  extend : 'Ext.chart.Chart',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.scheduler.quartz.SchedulerStatisticsType',
    'QDesk.store.quartz.stats.ExecStatDataStore',
    'QDesk.util.EnumUtils',

    'Ext.chart.theme.QuartzMostFrequentlyExecutedJobsChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  shadow : true,

  theme : 'QuartzMostFrequentlyExecutedJobsChartTheme',

  legend : {
    boxStroke : '#444444',
    labelColor : '#444444',
    position : 'right'
  },

  //insetPadding : 60,

  qdesk : {
    /**
     * Required config parameter with the statistics period length for this chart.
     */
    periodLength : undefined,

    /**
     * Required config parameter with the statistics period start date for this chart.
     */
    periodStart : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.periodLength )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodLength' );

    if ( !config.qdesk.periodStart )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodStart' );

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.quartz.stats.ExecStatDataStore' ),

      series : [
        {
          type : 'pie',
          field : 'countSuccessError',
          showInLegend : true,
          donut : false,
          tips : {
            dismissDelay : 0,
            trackMouse : true,
            renderer : function ( storeItem, item )
            {
              var html = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.stats.MostFrequentlyExecutedJobsChart.tooltipPieSection',
                  storeItem.get( 'jobGroupName' ),
                  storeItem.get( 'jobName' ),
                  Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countVeto' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'countSuccessError' ), '0,000' ) );

              this.update( html );
            }
          },
          highlight : {
            segment : {
              margin : 20
            }
          },
          label : {
            field : 'jobDisplayName',
            display : 'rotate',
            contrast : true,
            font : '15px Verdana'
          }
        }
      ]
    } );

    me.callParent( [config] );
  }
} );